/***************************************************************************
*   KBlocks, a falling blocks game by KDE                                *
*   SPDX-FileCopyrightText: 2010 Zhongjie Cai <squall.leonhart.cai@gmail.com>       *
*                                                                         *
*   SPDX-License-Identifier: GPL-2.0-or-later
***************************************************************************/
#include "KBlocksDefine.h"

const char *KBlocksRecordText[RecordDataType_Max_Count] = {
    "GameCount",
    "GameSeed",
    "Skipped",
    "MovePieceLeft",
    "MovePieceRight",
    "MovePieceUp",
    "MovePieceDown",
    "RotatePieceCW",
    "RotatePieceCCW",
    "GameOneStep",
    "PunishLineCount",
    "PunishLineSeed",
};
