/******************************************************************************
*   KBlocks, a falling blocks game by KDE                                     *
*   SPDX-FileCopyrightText: 2010-2021 Julian Helfferich <julian.helfferich@mailbox.org> *
*                                                                             *
*   SPDX-License-Identifier: GPL-2.0-or-later
******************************************************************************/
#include "GameLogicInterface.h"

GameLogicInterface::GameLogicInterface()
    : maGameList(nullptr)
{
}

#include "moc_GameLogicInterface.cpp"
