/******************************************************************************
*   KBlocks, a falling blocks game by KDE                                     *
*   SPDX-FileCopyrightText: 2009-2021 Julian Helfferich <julian.helfferich@mailbox.org> *
*                                                                             *
*   SPDX-License-Identifier: GPL-2.0-or-later
******************************************************************************/

#include "SceneInterface.h"

SceneInterface::SceneInterface()
    : QGraphicsScene()
{
}

#include "moc_SceneInterface.cpp"
